@@ -6,7 +6,7 @@ use warnings;
 
 require Exporter;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 $VERSION = eval $VERSION;  # see L<perlmodstyle>
 
 our @ISA = qw(Exporter);
@@ -21,7 +21,8 @@ our @ISA = qw(Exporter);
 our %EXPORT_TAGS = ( 'all' => [ qw(
 	iodef_descriptions iodef_assessments iodef_confidence iodef_impacts iodef_impacts_first
     iodef_additional_data iodef_systems iodef_addresses iodef_events_additional_data iodef_services
-    iodef_systems_additional_data iodef_normalize_restriction
+    iodef_systems_additional_data iodef_normalize_restriction iodef_guid iodef_uuid iodef_malware
+    iodef_systems_bgp
 ) ] );
 
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
@@ -77,7 +78,13 @@ sub iodef_descriptions {
 sub iodef_confidence {
     my $iodef = shift;
     
-    my $ret = iodef_assessments($iodef);
+    if(ref($iodef) eq 'IODEFDocumentType'){
+        $iodef = $iodef->get_Incident();
+    }
+    
+    $iodef = [$iodef] unless(ref($iodef) eq 'ARRAY');
+    
+    my $ret = @{$iodef}[0]->get_Assessment();
     my @array;
     foreach my $a (@$ret){
         push(@array,$a->get_Confidence());
@@ -88,57 +95,60 @@ sub iodef_confidence {
 sub iodef_assessments {
     my $iodef = shift;
     
-    return [] unless(ref($iodef) eq 'IODEFDocumentType');
-
-    my @array;
-    foreach my $i (@{$iodef->get_Incident()}){
-        unless(ref($i) eq 'IncidentType'){
-            warn 'invalid type: '.ref($i) if($::debug);
-            return [];
-        }
-        push (@array,@{$i->get_Assessment()});
-    }
-    return(\@array);
+    return [] unless(ref($iodef) eq 'IncidentType');
+    return $iodef->get_Assessment();
 }
 
 sub iodef_impacts {
     my $iodef = shift;
     
-    return [] unless(ref($iodef) eq 'IODEFDocumentType');
+    if(ref($iodef) eq 'IODEFDocumentType'){
+        $iodef = $iodef->get_Incident();
+    };
+    $iodef = [$iodef] unless(ref($iodef) eq 'ARRAY');
     
-    my $assessments = iodef_assessments($iodef);
-    my @array;
-    
-    foreach my $a (@$assessments){
-        unless(ref($a) eq 'AssessmentType'){
-            warn 'invalid type: '.ref($a) if($::debug);
-            return [];
-        }
-        push(@array,@{$a->get_Impact()});
+    my $array;
+    foreach my $i (@$iodef){
+        my @local = @{$i->get_Assessment()};
+        push(@$array, map { @{$_->get_Impact()} } @local);
     }
-    return(\@array);
+    return $array;
 }
 
 sub iodef_impacts_first {
     my $iodef = shift;
 
-    return [] unless(ref($iodef) eq 'IODEFDocumentType');
+    return [] unless(ref($iodef) eq 'IncidentType');
 
     my $impacts = iodef_impacts($iodef);
     my $impact = @{$impacts}[0];
     return($impact);
 }
 
-sub iodef_addresses {
-    my $iodef = shift;
+sub iodef_address_type {
     my $type = shift;
     
+    # TODO -- return contstant for net-addr-ipv4, etc.
+    
+    return $type;   
+}
+
+sub iodef_addresses {
+    my $iodef   = shift;
+    my $type    = shift;
+    
+    return unless($iodef);
+    
     $type = iodef_address_type($type) if($type);
     
-    return [] unless(ref($iodef) eq 'IODEFDocumentType');
+    if(ref($iodef) eq 'IODEFDocumentType'){
+        $iodef = $iodef->get_Incident();
+    }
+    
+    $iodef = [$iodef] unless(ref($iodef) eq 'ARRAY');
         
     my @array;
-    foreach my $i (@{$iodef->get_Incident()}){
+    foreach my $i (@$iodef){
         next unless($i->get_EventData());
         foreach my $e (@{$i->get_EventData()}){
             my @flows = (ref($e->get_Flow()) eq 'ARRAY') ? @{$e->get_Flow()} : $e->get_Flow();
@@ -158,42 +168,16 @@ sub iodef_addresses {
     return(\@array);
 }
 
-sub iodef_services {
-    my $iodef = shift;
-    
-    my @array;
-    foreach my $i (@{$iodef->get_Incident()}){
-        next unless($i->get_EventData());
-        foreach my $e (@{$i->get_EventData()}){
-            my @flows = (ref($e->get_Flow()) eq 'ARRAY') ? @{$e->get_Flow()} : $e->get_Flow();
-            foreach my $f (@flows){
-                my @systems = (ref($f->get_System()) eq 'ARRAY') ? @{$f->get_System()} : $f->get_System();
-                foreach my $s (@systems){
-                    my $services = $s->get_Service();
-                    $services = [$services] unless(ref($services) eq 'ARRAY');
-                    foreach my $svc (@$services){
-                        $svc = [$svc] unless(ref($svc) eq 'ARRAY');
-                        push(@array,@$svc);
-                    }
-                }
-            }
-        }
-    }
-    return(\@array);
-}
-
 sub iodef_systems {
     my $iodef = shift;
     
     my @array;
-    foreach my $i (@{$iodef->get_Incident()}){
-        next unless($i->get_EventData());
-        foreach my $e (@{$i->get_EventData()}){
-            my @flows = (ref($e->get_Flow()) eq 'ARRAY') ? @{$e->get_Flow()} : $e->get_Flow();
-            foreach my $f (@flows){
-                my @systems = (ref($f->get_System()) eq 'ARRAY') ? @{$f->get_System()} : $f->get_System();
-                push(@array,@systems);
-            }
+    return unless($iodef->get_EventData());
+    foreach my $e (@{$iodef->get_EventData()}){
+        my @flows = (ref($e->get_Flow()) eq 'ARRAY') ? @{$e->get_Flow()} : $e->get_Flow();
+        foreach my $f (@flows){
+            my @systems = (ref($f->get_System()) eq 'ARRAY') ? @{$f->get_System()} : $f->get_System();
+            push(@array,@systems);
         }
     }
     return(\@array);
@@ -202,9 +186,10 @@ sub iodef_systems {
 sub _additional_data {
     my $array = shift;
     
+    $array = [$array] unless(ref($array) eq 'ARRAY'); 
     my $return_array;
     foreach my $e (@$array){
-        next unless($e->get_AdditionalData());
+        next unless($e && $e->get_AdditionalData());
         my @additional_data = (ref($e->get_AdditionalData()) eq 'ARRAY') ? @{$e->get_AdditionalData()} : $e->get_AdditionalData();
         push(@$return_array,@additional_data);
     }
@@ -213,19 +198,32 @@ sub _additional_data {
 
 sub iodef_additional_data {
     my $iodef = shift;
+
+    if(ref($iodef) eq 'IODEFDocumentType'){
+        $iodef = $iodef->get_Incident();
+    }
+    $iodef = [$iodef] unless(ref($iodef) eq 'ARRAY');
     
-    return [] unless(ref($iodef) eq 'IODEFDocumentType');
-    #my @array;
-    #foreach my $i (@{$iodef->get_Incident()}){
-    #    next unless($i->get_AdditionalData());
-    #    my @additional_data = (ref($i->get_AdditionalData()) eq 'ARRAY') ? @{$i->get_AdditionalData()} : $i->get_AdditionalData();
-    #    push(@array,@additional_data);
-    #}
-    
-    my $array = _additional_data($iodef->get_Incident());
+    my $array = _additional_data($iodef);
     return($array);
 }
 
+sub iodef_malware {
+    my $iodef = shift;
+
+    my $ad = iodef_additional_data($iodef);
+    return unless($#{$ad});
+    
+    my $array = [];
+    foreach (@$ad){
+        next unless($_->get_meaning() =~ /^malware hash$/);
+        push(@$array,$_);
+    }
+    return unless($#{$array} > -1);
+    return $array;
+}
+    
+
 sub iodef_events_additional_data {
     my $iodef = shift;
     
@@ -235,10 +233,6 @@ sub iodef_events_additional_data {
         if(my $ret = _additional_data($i->get_EventData())){
             push(@$array,@$ret);
         }
-        #foreach my $e (@{$i->get_EventData()}){
-        #    my @additional_data = (ref($e->get_AdditionalData()) eq 'ARRAY') ? @{e->get_AdditionalData()} : $e->get_AdditionalData();
-        #    push(@array,@additional_data);
-        #}
     }
     
     return($array);
@@ -247,29 +241,56 @@ sub iodef_events_additional_data {
 sub iodef_systems_additional_data {
     my $iodef = shift;
     
+    return unless(ref($iodef) eq 'IncidentType');
+    
     my $array;
-    foreach my $i (@{$iodef->get_Incident()}){
-        next unless($i->get_EventData());
-        foreach my $e (@{$i->get_EventData()}){
-            my @flows = (ref($e->get_Flow()) eq 'ARRAY') ? @{$e->get_Flow()} : $e->get_Flow();
-            foreach my $f (@flows){
-                next unless($f->get_System());
-                my @systems = (ref($f->get_System()) eq 'ARRAY') ? @{$f->get_System()} : $f->get_System();
-                if(my $ret = _additional_data($f->get_System())){
-                    push(@$array,@$ret);
-                }
-                #foreach my $s (@systems){
-                #    next unless($s->get_AdditionalData());
-                #    my @additional_data = (ref($s->get_AdditionalData()) eq 'ARRAY') ? @{$s->get_AdditionalData()} : $s->get_AdditionalData();
-                #    next unless($#additional_data > -1);
-                #    push(@array,@additional_data);
-                #}
+    foreach my $e (@{$iodef->get_EventData()}){
+        my @flows = (ref($e->get_Flow()) eq 'ARRAY') ? @{$e->get_Flow()} : $e->get_Flow();
+        foreach my $f (@flows){
+            next unless($f->get_System());
+            my @systems = (ref($f->get_System()) eq 'ARRAY') ? @{$f->get_System()} : $f->get_System();
+            if(my $ret = _additional_data($f->get_System())){
+                push(@$array,@$ret);
             }
         }
     }
     return($array);
 }
 
+sub iodef_systems_bgp {
+    my $iodef = shift;
+    
+    return unless(ref($iodef) eq 'SystemType');
+    my $array = _additional_data($iodef) or return;
+    
+    my $hash;
+    
+    foreach my $s (@$array){
+        next unless(lc($s->get_meaning()) =~ /^(asn|asn_desc|prefix|rir|cc)$/);
+        $hash->{$s->get_meaning()} = $s->get_content(); 
+    }
+    return($hash);
+}
+    
+
+sub iodef_guid {
+    my $iodef = shift;
+    
+    return unless(ref($iodef) eq 'IncidentType');
+    
+    my $ad = $iodef->get_AdditionalData();
+    foreach (@$ad){
+        next unless($_->get_meaning() =~ /^guid/);
+        ## TODO -- return array of guids?
+        return $_->get_content();
+    }
+}
+
+sub iodef_uuid {
+    my $iodef = shift;
+    return $iodef->get_IncidentID->get_content();
+}
+
 sub new {
     my $class = shift;
     my $args = shift;
