@@ -22,6 +22,23 @@ sub process {
     
     my @additional_data ;
     
+    # in the event that we wanna track this and pass it along
+    my $service;
+    if($data->{'service'} && ref($data->{'service'}) eq 'ServiceType'){
+        $service = $data->{'service'};
+    } elsif($data->{'protocol'} || $data->{'portlist'}) {
+        $service = ServiceType->new();
+        my $proto = $data->{'protocol'} || $data->{'ip_protocol'};
+        if($data->{'portlist'}){
+            $service->set_Portlist($data->{'portlist'});
+            # IODEF requires a default
+            $proto = 'tcp' unless($proto);
+        }
+        if($proto){
+            $proto = $self->normalize_protocol($proto);
+            $service->set_ip_protocol($proto);
+        }
+    }
     
     my $system = SystemType->new({
         Node    => NodeType->new({
@@ -44,6 +61,10 @@ sub process {
         );
     }
     
+    if($service){
+        $system->set_Service($service);
+    }
+    
     my $event = EventDataType->new({
         Flow    => FlowType->new({
             System  => $system,
